/*

	SafeString

	Author:	Vendicator
	
	Description:
	The purpose of this class is to make a STL conformant string class
	using TCHAR and the safe string operation provided by strsafe.h

	Changelog:

	[2003-06-12 - Vendicator]
	- First work, added assign(), append(), c_str().
	
*/

#ifndef __SAFESTRING_H
#define __SAFESTRING_H

#include <windows.h>

#define STRSAFE_NO_DEPRECATE
#include <strsafe.h>

#define MAX_STRING_LENGTH STRSAFE_MAX_CCH - 1
#define MAX_NUMBER_LENGTH 64

#ifndef _T
#define _T(str) TEXT(str)
#endif

//#define __STL

#ifndef __STL

class SafeString
{
public:
	SafeString() :
		nLength(0),
		nSize(0),
		str_data(NULL),
		str_char_data(NULL)
	{
	}

	SafeString(LPCTSTR str) :
		nLength(0),
		nSize(0),
		str_data(NULL),
		str_char_data(NULL)
	{
		assign(str);
	}

	SafeString(size_t size) :
		nLength(0),
		nSize(size),
		str_data(NULL),
		str_char_data(NULL)		
	{
		AllocateSize(size);
		size_t byte_length;
		
		if ( SUCCEEDED(StringCbLength(str_data, size * sizeof(TCHAR), &byte_length)) )
			memset(str_data, 0, byte_length);
		else
		{
			Cleanup();
		}
	}

	~SafeString()
	{
		Cleanup();
	}

	const SafeString& operator=(LPCTSTR str)
	{
		assign(str);
		return *this;
	}
	
	void assign(LPCTSTR str, size_t copy_length = 0)
	{
		size_t max_length = MAX_STRING_LENGTH;
		size_t length;
		HRESULT ok = S_OK;

		Cleanup();

		ok = StringCchLength(str, max_length, &length);

		if ( SUCCEEDED(ok) )
		{
			if ( copy_length > 0 )
			{
				// don't try to copy more than the inputed string length
				length = min(copy_length, length);
				AllocateSize(length);
				StringCchCopyN(str_data, length+1, str, length);
			}
			else
			{
				AllocateSize(length);
				StringCchCopy(str_data, length+1, str);
			}

			nLength = length;
		}
	}

	void append(int number)
	{
		TCHAR buffer[MAX_NUMBER_LENGTH+1];
		StringCchPrintf(buffer, MAX_NUMBER_LENGTH, _T("%d"), number);
		append(buffer);
	}

	void append(LPCTSTR str)
	{
		LPTSTR new_str;

		size_t add_length;
		size_t new_length;

		if ( SUCCEEDED(StringCchLength(str, MAX_STRING_LENGTH, &add_length)) )
		{
			new_length = nLength + add_length;

			// if we need a bigger string container
			if (new_length > nSize)
			{
				// copy old string to new container
				AllocateSize(new_length, &new_str);
				// if there was something in the string before
				if (nLength > 0)
					StringCchCopy(new_str, new_length+1, str_data);
				
				// free old memory
				Cleanup();

				// set up new container
				nLength = new_length;
				nSize = new_length;
				str_data = new_str;
			}

			// concatenate with adding string
			StringCbCat(str_data, new_length+1, str);
		}
	}

	size_t length()
	{
		return nLength;
	}

	// either tchar string returned
	LPCTSTR c_str() const
	{
		return str_data;
	}

	// conversion to real char string
	LPSTR char_str()
	{
		if (str_char_data != NULL)
			delete [] str_char_data;

		str_char_data = new char[nLength + 1];

		for (size_t i = 0; i < nLength; i++)
		{
			str_char_data[i] = (char) str_data[i];
		}
		str_char_data[nLength] = '\0';
		
		return str_char_data;
	}

	// sprintf assignment
	void formated_assign(LPCTSTR pszFormat, ...)
	{
		LPTSTR buffer;
		AllocateSize(MAX_STRING_LENGTH, &buffer);

		va_list argList;
		va_start(argList, pszFormat);
		StringCchPrintf(buffer, MAX_STRING_LENGTH, pszFormat, argList);
		va_end(argList);
		
		assign(buffer);
	}

protected:
	// frees memory for the string and any converted char[]
	void Cleanup()
	{
		if (str_data != NULL)
		{
			delete [] str_data;
			str_data = NULL;
		}
		if (str_char_data != NULL)
		{
			delete [] str_char_data;
			str_char_data = NULL;	
		}
		nLength = 0;
		nSize = 0;
	}

	// allocate size for a certain amount of chars
	void AllocateSize(size_t size, LPTSTR* str = NULL)
	{
		if (str == NULL)
		{
			nSize = size + 1;
			str_data = new TCHAR[size + 1];
		}
		else
		{
			*str = new TCHAR[size + 1];
		}		
	}

protected:
	size_t nLength;
	size_t nSize;
	LPTSTR str_data;

	LPSTR str_char_data;

};

#else

#include <string>
using std::string;
typedef string SafeString;

#endif //__STL

#endif